import time
from datetime import datetime
import re
import subprocess
import os


import sdi_utils.gensolution as gs
import sdi_utils.set_logging as slog
import sdi_utils.textfield_parser as tfp
import sdi_utils.tprogress as tp

try:
    api
except NameError:
    class api:
        class Message:
            def __init__(self,body = None,attributes = ""):
                self.body = body
                self.attributes = attributes
                
        def send(port,msg) :
            pass
            if port == outports[2]['name'] :
                print(msg.attributes)
                print(msg.body)


        def set_config(config):
            api.config = config
    
        class config:
            ## Meta data
            config_params = dict()
            tags = {'sdi_utils':''}
            version = "0.0.1"
            operator_name = 'filter_date'
            operator_description = "Filter date string"
            operator_description_long = "Filters the names with today substring."
            add_readme = dict()
            debug_mode = True
            config_params['debug_mode'] = {'title': 'Debug mode',
                                           'description': 'Sending debug level information to log port',
                                           'type': 'boolean'}
            start_date = 'None'
            config_params['start_date'] = {'title': 'Start Date',
                                           'description': 'Start date. If value is none then today date is taken. Format:YYYY-MM-DD',
                                           'type': 'string'}
            end_date = 'None'
            config_params['end_date'] = {'title': 'End Date',
                                           'description': 'End date. If value is none then today date is taken. Format:YYYY-MM-DD',
                                           'type': 'string'}


count = 0

def process(msg):
    global count
    att_dict = msg.attributes

    att_dict['operator'] = 'filter_today'
    logger, log_stream = slog.set_logging(att_dict['operator'], loglevel=api.config.debug_mode)
    logger.info("Process started")
    time_monitor = tp.progress()
    today_str = time.strftime("%Y-%m-%d")
    logger.debug("Today string: {}".format(today_str))

    start_date_str = tfp.read_value(api.config.start_date)
    if start_date_str == None :
        start_date_str = today_str
    start_date = datetime.strptime(start_date_str, "%Y-%m-%d")

    end_date_str = tfp.read_value(api.config.end_date)
    if end_date_str == None :
        end_date_str = today_str
    end_date = datetime.strptime(end_date_str, "%Y-%m-%d")

    filename = msg.attributes['file']['path']
    logger.debug('Attributes: {}'.format(str(msg.attributes)))
    logger.debug('Body: {}'.format(str(msg.body)))
    logger.info('Filename: {}'.format(filename))

    date_found = re.search('\d{4}-\d{2}-\d{2}',filename)
    if date_found :
        file_date_str = date_found.group(0)
        logger.info('Date string found in filename: {}'.format(file_date_str))
        file_date = datetime.strptime(file_date_str, "%Y-%m-%d")
        if file_date >= start_date and file_date <= end_date :
            count +=1
            logger.info('Date in filename is within given range: {} ({} - {})'.format(file_date_str,start_date_str,end_date_str))
            fmsg = api.Message(attributes = msg.attributes, body=filename)
            api.send(outports[2]['name'],fmsg)
            if msg.attributes['message.lastBatch'] :
                api.send(outports[1]['name'],count)
                logger.info('Number of files: {}'.format(count))
    else :
        logger.info('No date pattern found in filename: {}'.format(filename))

    api.send(outports[0]['name'], log_stream.getvalue())


inports = [{'name': 'input', 'type': 'message.file',"description":"Filename"}]
outports = [{'name': 'log', 'type': 'string',"description":"Logging data"},\
            {'name': 'numfiles', 'type': 'int64',"description":"Number of files"},\
            {'name': 'output', 'type': 'message.file',"description":"Filename"}]


#api.set_port_callback(inports[0]['name'], process)

def test_operator() :
    config = api.config
    config.debug_mode = True
    config.start_date = '2020-02-01'
    config.end_date = '2020-03-31'
    api.set_config(config)

    in_dir = '/Users/Shared/data/onlinemedia/crawled_texts'
    files_in_dir = [f for f in os.listdir(in_dir) if os.path.isfile(os.path.join(in_dir, f)) and re.match('.*json', f)]
    for i, fname in enumerate(files_in_dir):
        fbase = fname.split('.')[0]
        eos = True if len(files_in_dir) == i + 1 else False
        attributes = {'format': 'csv', "storage.filename": fbase, 'storage.endOfSequence': eos, \
                      'storage.fileIndex': i, 'storage.fileCount': len(files_in_dir),'process_list':[]}
        attributes = {'file':{'path':fname}, 'message.lastBatch':eos}
        msg = api.Message(attributes=attributes, body=fname)
        process(msg)


if __name__ == '__main__':
    test_operator()

    if True :
        gs.gensolution(os.path.realpath(__file__), api.config, inports, outports)
        solution_name = api.config.operator_name+'_'+api.config.version

        subprocess.run(["vctl", "solution", "bundle", '/Users/d051079/OneDrive - SAP SE/GitHub/sdi_utils/solution/operators/sdi_utils_operators_0.0.1',\
                                  "-t", solution_name])
        subprocess.run(["mv", solution_name+'.zip', '../../../solution/operators'])



import subprocess
import os

import sdi_utils.gensolution as gs
import sdi_utils.set_logging as slog
import sdi_utils.textfield_parser as tfp
import sdi_utils.tprogress as tp

try:
    api
except NameError:
    class api:
        class Message:
            def __init__(self, body=None, attributes=""):
                self.body = body
                self.attributes = attributes

        def send(port, msg):
            if isinstance(msg, api.Message):
                print('Attributes\n{}: {}'.format(port, msg.attributes))
                print('Body\{}: {}'.format(port, msg.body))
            else:
                print('{}: {}'.format(port, msg))

        def call(config):
            api.config = config
            process()

        class config:
            ## Meta data
            config_params = dict()
            tags = {'pandas': '', 'sdi_utils': '', 'scrapy':''}
            version = "0.0.1"
            operator_description = "scrapy executor"
            operator_description_long = "Starts scrapy and sends output to data port and stderr to log port."
            add_readme = dict()

            debug_mode = True
            config_params['debug_mode'] = {'title': 'Debug mode',
                                           'description': 'Sending debug level information to log port',
                                           'type': 'boolean'}
            cmd = 'None'
            config_params['cmd'] = {'title': 'Command', 'description': \
                'Command to be executed', 'type': 'string'}

            scrapy_dir = 'None'
            config_params['scrapy_dir'] = {'title': 'Scrapy Directory', 'description': \
                'Scrapy directory on container', 'type': 'string'}

inports = [{"name": "trigger", "type": "string", "description": "Trigger"}]
outports = [{'name': 'log', 'type': 'string', "description": "Logging"},\
            {'name': 'stderr', 'type': 'string', "description": "Stderr"}, \
            {"name": "data", "type": "message", "description": "data"}]

def process(str) :

    logger, log_stream = slog.set_logging('scrapy_executor',loglevel=api.config.debug_mode)

    scrapy_dir = tfp.read_value(api.config.scrapy_dir)
    if scrapy_dir :
        logger.info('Change directory to: {}'.format(scrapy_dir))
        os.chdir(scrapy_dir)

    cmd = tfp.read_list(api.config.cmd,sep=' ')
    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE,stderr=subprocess.PIPE,universal_newlines=True)

    output = '1'
    errout = '1'
    while proc.poll() is None or  output != '' or  errout != '':
        output = proc.stdout.readline()
        errout = proc.stderr.readline()
        if output != '':
            att_dict = {'article':'test'}
            msg = api.Message(attributes=att_dict,body=output)
            api.send(outports[2]['name'],msg)
        if errout != '':
            api.send(outports[1]['name'],errout)

#api.set_port_callback(inports[0]['name'], process)


def main():
    print('CWD: {}'.format(os.getcwd()))
    config = api.config
    config.debug_mode = True
    config.scrapy_dir = '/Users/d051079/OneDrive - SAP SE/GitHub/sdi_utils/src/sdi_utils_operators/scrapy_executor'
    config.cmd = 'python /tmp/outputgen.py'
    api.call(config)

if __name__ == '__main__':
    main()






try:
    api
except NameError:
    class api:
        class Message:
            def __init__(self, body=None, attributes=""):
                self.body = body
                self.attributes = attributes

        class config:
            ## Meta data
            config_params = dict()
            tags = {}
            version = "0.0.1"
            operator_description = "AND Gate"
            operator_description_long = "Triggers when input on both input ports."
            add_readme = dict()

inports = [{"name": "input1", "type": "any", "description": "Trigger, content not used"}, \
           {"name": "input2", "type": "any", "description": "Trigger, content not used"}]
outports = [{'name': 'trigger', 'type': 'string', "description": "Trigger next process step"}]


def trigger(msg1,msg2):
    api.send(outports[0]['name'], "Get on")


#api.set_port_callback([inports[0]['name'],inports[1]['name']], trigger)







import csv

import sdi_utils.set_logging as slog
import sdi_utils.textfield_parser as tfp
import sdi_utils.tprogress as tp


try:
    api
except NameError:
    class api:
        class Message:
            def __init__(self, body=None, attributes=""):
                self.body = body
                self.attributes = attributes

        def send(port, msg):
            if port == outports[1]['name']:
                print('PORT: {}'.format(port))
                print(msg.attributes)
                print(msg.body)

        def set_config(config):
            api.config = config

        def set_port_callback(port, callback):
            pass

        class config:
            ## Meta data
            config_params = dict()
            tags = {'sdi_utils': ''}
            version = "0.0.1"
            operator_description = "Pivotize table"
            operator_description_long = "Sends input topics to SQL processor and topic frequency operator."
            add_readme = dict()

            debug_mode = True
            config_params['debug_mode'] = {'title': 'Debug mode',
                                           'description': 'Sending debug level information to log port',
                                           'type': 'boolean'}
            ref_col_index = 0
            config_params['ref_col_index'] = {'title': 'Reference column index',
                                           'description': 'Reference column index',
                                           'type': 'integer'}
            pivot_cols_index = '2, 3'
            config_params['pivot_cols_index'] = {'title': 'Pivot columns index list',
                                           'description': 'Pivot columns index list',
                                           'type': 'string'}

            pivot_column_name = 'PIVOT_COLUMN'
            config_params['pivot_column_name'] = {'title': 'New Colum Name',
                                                 'description': 'New Colum Name',
                                                 'type': 'string'}

def process(msg):
    logger, log_stream = slog.set_logging('pivotize', loglevel=api.config.debug_mode)
    logger.info("Process started")
    time_monitor = tp.progress()

    table = msg.body

    pivot_col_name = api.config.pivot_column_name
    ref_col_index = api.config.ref_col_index
    pivot_cols = tfp.read_list(api.config.pivot_cols_index,test_number=True)
    for i in pivot_cols :
        if not isinstance(i,int) :
            raise Exception('Config parameter needs to be list of integers')

    logger.debug('Table to pivot: {} - {} with {} columns'.format(len(table),len(table[0]),len(pivot_cols)))

    table = msg.body
    att_dict = msg.attributes
    ref_column = att_dict['table']['columns'][ref_col_index]
    pivot_col = att_dict['table']['columns'][pivot_cols[0]]
    pivot_col['name'] = pivot_col_name
    att_dict['table']['columns'] = [ref_column,pivot_col]

    pivot_table = list()
    for row in table :
        pivot_table.extend([ [row[ref_col_index],row[c]] for c in pivot_cols ])

    # remove empty rows
    pivot_table = [r for r in pivot_table if not r[1] == '' and not r[1] == None]

    logger.debug('Pivot Table: {} - {}'.format(len(pivot_table), len(pivot_table[0])))
    logger.debug('Process ended, rows processed {}  - {}  '.format(len(table), time_monitor.elapsed_time()))
    msg = api.Message(attributes=att_dict,body = pivot_table)
    api.send(outports[1]['name'], msg)
    api.send(outports[0]['name'], log_stream.getvalue())


outports = [{'name': 'log', 'type': 'string', "description": "Logging data"},
            {'name': 'table', 'type': 'message.table', "description": "Message with pivotized table"}]
inports = [{'name': 'topic', 'type': 'message.table', "description": "Message with body as table."}]


#api.set_port_callback(inports[0]['name'], process)


def main():
    topics_filename = '/Users/Shared/data/onlinemedia/repository/topics.csv'
    topics = list()
    with open(topics_filename, mode='r') as csv_file:
        rows = csv.reader(csv_file, delimiter=',')
        next(rows,None)
        for r in rows:
            topics.append(r)

    attributes = {"table":{"columns":[{"class":"string","name":"TOPIC","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"LANGUAGE","nullable":True,"size":3,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"TYPE","nullable":True,"size":10,"type":{"hana":"NVARCHAR"}},
                                      {"class":"timestamp","name":"DATE","nullable":True,"type":{"hana":"DATE"}},
                                      {"class":"timestamp","name":"EXPERIY_DATE","nullable":True,"type":{"hana":"DATE"}},
                                      {"class":"string","name":"ATTRIBUTE","nullable":True,"size":15,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_1","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_2","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_3","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_4","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_5","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_6","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_7","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_8","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_9","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}},
                                      {"class":"string","name":"KEYWORD_10","nullable":True,"size":80,"type":{"hana":"NVARCHAR"}}],
                           "name":"DIPROJECTS.TOPICS","version":1}}
    topics_msg = api.Message(attributes=attributes, body=topics)
    config = api.config
    config.ref_col_index = 0
    config.pivot_cols_index = "6,7,8,9,10,11,12,13,14,15"
    config.debug_mode = True
    api.set_config(config)
    process(topics_msg)

if __name__ == '__main__':
    main()
    # gs.gensolution(os.path.realpath(__file__), config, inports, outports)








import logging
from io import StringIO



### set logger for strerr and to StringIO
def set_logging(name='operator',loglevel = logging.INFO, stream_output = True) :

    # to prevent the necessity to import logging only for passing the loglevel a string argument is

    log_level = logging.INFO
    if isinstance(loglevel,str) :
        try :
            log_level = map_logs[loglevel]
        except ValueError :
            raise ValueError('Unknown logging level. Valid values: INFO, DEBUG, WARNING, ERROR but not {}'.format(loglevel))
    elif isinstance(loglevel, bool) and loglevel == True  :
        log_level = logging.DEBUG
    elif isinstance(loglevel, int) :
        log_level = loglevel
    else:
        raise ValueError('Unknown logging level {}'.format(loglevel))

    format = '%(asctime)s ;  %(levelname)s ; %(name)s ; %(message)s'
    logging.basicConfig(level=log_level,format=format,datefmt='%H:%M:%S')

    logger = logging.getLogger(name=name)
    logger.setLevel(loglevel)
    log_stream = StringIO()
    # stream
    if stream_output :
        sh = logging.StreamHandler(stream=log_stream)
        sh.setFormatter(logging.Formatter(format, datefmt='%H:%M:%S'))
        sh.setLevel(log_level)
        logger.addHandler(sh)
        logger.info('Logger setup')
    logger.info('Logging Level: {}'.format(log_level))

    # stderr
    #if stderr_output :
    #    eh = logging.StreamHandler(stream=sys.stderr)
    #    eh.setFormatter(logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s', datefmt='%H:%M:%S'))
    #    logger.addHandler(eh)

    return logger, log_stream

def main() :
    logger, log_stream = set_logging(name = 'Test', loglevel=True)
    logger.debug('Message debug level')
    for h in logger.handlers :
        h.setLevel(logging.DEBUG)
        logger.setLevel(logging.DEBUG)
        #logger.addHandler(h)
    logger.debug('Message debug level')
    logger.info('Message info level')

    print('Log String: ')
    print(log_stream.getvalue())

if __name__ == '__main__':
    main()




import io
import subprocess
import os
import csv

import sdi_utils.gensolution as gs
import sdi_utils.set_logging as slog
import sdi_utils.textfield_parser as tfp
import sdi_utils.tprogress as tp

try:
    api
except NameError:
    class api:
        class Message:
            def __init__(self, body=None, attributes=""):
                self.body = body
                self.attributes = attributes

        def set_config(config):
            api.config = config

        def send(port, msg):
            if port == outports[1]['name']:
                print(msg.attributes)
                for i, dk in enumerate(msg.body):
                    print(dk)
                    if i > 10:
                        break
                header = [c["name"] for c in msg.attributes['table']['columns']]
                print(header)

        class config:
            ## Meta data
            config_params = dict()
            tags = {'sdi_utils': ''}
            version = "0.0.1"
            operator_description = "csv to table"
            operator_name = 'csv_table'
            operator_description_long = "Converts csv to table"
            add_readme = dict()
            debug_mode = True
            config_params['debug_mode'] = {'title': 'Debug mode',
                                           'description': 'Sending debug level information to log port',
                                           'type': 'boolean'}
            collect = True
            config_params['collect'] = {'title': 'Collect data',
                                        'description': 'Collect data before sending it to the output port',
                                        'type': 'boolean'}
            has_header = True
            config_params['has_header'] = {'title': 'Has_header',
                                           'description': 'If csv-file has a header.',
                                           'type': 'boolean'}
            separator = ';'
            config_params['separator'] = {'title': 'Separator', 'description': 'Separator',
                                          'type': 'string'}

result_table = list()


def process(msg):
    att_dict = msg.attributes

    global result_table

    # ONLY DUE TO BUG
    changed_debug = False
    changed_header = False
    if api.config.debug_mode == False:
        api.config.debug_mode = True
        changed_debug = True
    if api.config.has_header == False:
        api.config.has_header = True
        changed_header = True

    att_dict['operator'] = 'csv_to_table'
    logger, log_stream = slog.set_logging(att_dict['operator'], loglevel=api.config.debug_mode)

    # ONLY DUE TO BUG
    if changed_debug:
        logger.debug('Logger level changed to DEBUG')
    if changed_header:
        logger.debug('has_header changed to True')

    logger.info("Process started")
    time_monitor = tp.progress()

    logger.debug('Attributes: {}'.format(str(att_dict)))

    str_body = msg.body.decode('utf-8')
    csv_io = io.StringIO(str_body)

    rows = csv.reader(csv_io, delimiter=api.config.separator)
    # first row
    row1 = next(rows)
    if not api.config.has_header:
        result_table.append(row1)
    for r in rows:
        result_table.append(r)

    att_dict["table"] = {"name": msg.attributes['file']['path'], "version": 1, "columns": list()}
    for i, h in enumerate(row1):
        if api.config.has_header:
            att_dict["table"]["columns"].append({"name": h})
        else:
            att_dict["table"]["columns"].append({"name": 'column_' + str(i)})

    logger.debug('Process ended: {}'.format(time_monitor.elapsed_time()))
    out_msg = api.Message(attributes=att_dict, body=result_table)
    if 'message.lastBatch' in msg.attributes:
        if msg.attributes['message.lastBatch'] or api.config.collect == False:
            api.send(outports[1]['name'], out_msg)
            result_table = list()
    else:
        api.send(outports[1]['name'], out_msg)

    if api.config.debug_mode:
        for i, dk in enumerate(result_table):
            logger.debug(dk)
            if i > 10:
                break

    api.send(outports[0]['name'], log_stream.getvalue())


inports = [{'name': 'stream', 'type': 'message.file', "description": "Input csv byte or string"}]
outports = [{'name': 'log', 'type': 'string', "description": "Logging data"}, \
            {'name': 'data', 'type': 'message.table', "description": "Output data as table"}]

api.set_port_callback(inports[0]['name'], process)


def test_operator():
    config = api.config
    config.debug_mode = True
    config.collect = False
    config.separator = ','
    config.has_header = False
    api.set_config(config)

    fname = '/Users/Shared/data/onlinemedia/repository/lexicon.csv'

    fbase = fname.split('.')[0]
    attributes = {'format': 'csv', "file": {'path': fname}, 'message.lastBatch': True, \
                  'storage.fileIndex': 0, 'storage.fileCount': 1, 'process_list': []}
    csvstream = open(fname, mode='rb').read()
    msg = api.Message(attributes=attributes, body=csvstream)
    process(msg)


if __name__ == '__main__':
    test_operator()
    if True :
        gs.gensolution(os.path.realpath(__file__), api.config, inports, outports)
        solution_name = api.config.operator_name+'_'+api.config.version
        print('Solution name: {}'.format(solution_name))
        print('Current directory: {}'.format(os.getcwd()))
        subprocess.run(["vctl", "solution", "bundle", '/Users/d051079/OneDrive - SAP SE/GitHub/sdi_utils/solution/operators/sdi_utils_operators_0.0.1',\
                                  "-t", solution_name])
        subprocess.run(["mv", solution_name+'.zip', '../../../../solution/operators'])



import sys

import sdi_utils.gensolution as gs
import sdi_utils.set_logging as slog
import sdi_utils.textfield_parser as tfp
import sdi_utils.tprogress as tp

try:
    api
except NameError:
    class api:
        class Message:
            def __init__(self, body=None, attributes=""):
                self.body = body
                self.attributes = attributes

        def send(port, str):
            print('Send')
            print(str)

        def set_config(config):
            api.config = config

        class config:
            ## Meta data
            config_params = dict()
            tags = {'sdi_utils': ''}
            version = "0.0.1"
            operator_description = "String Collector"
            operator_description_long = "Converts lines to array"
            add_readme = dict()
            debug_mode = True
            config_params['debug_mode'] = {'title': 'Debug mode',
                                           'description': 'Sending debug level information to log port',
                                           'type': 'boolean'}
            threshold_size = 1000
            config_params['threshold_size'] = {'title': 'Threshold size',
                                        'description': 'Threshold by which string is send. ',
                                        'type': 'integer'}

collector_str = ''
logger, log_stream = slog.set_logging('string_collector', loglevel=api.config.debug_mode)

def collect(str):

    global collector_str
    global logger, log_stream

    collector_str += str
    mem = sys.getsizeof(collector_str)
    if mem > api.config.threshold_size * 1024 :
        api.send(outports[1]['name'],collector_str)
        logger.debug('String sent to output due to size: {}kB ({})'.format(mem/1024,api.config.threshold_size))
        api.send(outports[0]['name'],log_stream.getvalue())
        collector_str = ''

def trigger(trig) :
    global collector_str
    logger.debug('String sent to output due to trigger: {:.1f}kB ({})'.format(sys.getsizeof(collector_str)/ 1024, api.config.threshold_size))
    api.send(outports[0]['name'], log_stream.getvalue())
    api.send(outports[1]['name'],collector_str)
    collector_str = ''

inports = [{'name': 'string', 'type': 'string', "description": "Input string"},
           {'name': 'trigger', 'type': 'any', "description": "Trigger input"}]
outports = [{'name': 'log', 'type': 'string', "description": "Log strings"},
            {'name': 'string', 'type': 'string', "description": "Output collected string"}]


#api.set_port_callback(inports[0]['name'], collect)
#api.set_port_callback(inports[0]['name'], trigger)

def main():
    config = api.config
    config.debug_mode = True
    config.threshold_size = 1
    api.set_config(config)

    for i in range (0,105) :
        collect('text line: {}\n'.format(i))
    trigger('los')



if __name__ == '__main__':
    main()
    # gs.gensolution(os.path.realpath(__file__), config, inports, outports)




import re

import sdi_utils.gensolution as gs
import sdi_utils.set_logging as slog
import sdi_utils.textfield_parser as tfp
import sdi_utils.tprogress as tp

try:
    api
except NameError:
    class api:
        class Message:
            def __init__(self,body = None,attributes = ""):
                self.body = body
                self.attributes = attributes
                
        def send(port,str) :
            print('Port send: {}'.format(str))
            return str
    
        def call(config,str):
            api.config = config
            return process(str)
            
        def set_port_callback(port, callback) :
            pass
    
        class config:
            ## Meta data
            config_params = dict()
            tags = {'pandas': '','sdi_utils':''}
            version = "0.0.1"
            operator_description = "Parse Logs"
            operator_description_long = "Parse logs for keywords to control process flow, e.g. stop a branched pipeline consistently"
            add_readme = dict()

            debug_mode = True
            config_params['debug_mode'] = {'title': 'Debug mode',
                                           'description': 'Sending debug level information to log port',
                                           'type': 'boolean'}
            marker_map = 'operator_name1: keyword1 ; operator_name2: keyword2'
            config_params['marker_map'] = {'title': 'List of marker','description': \
                'List of keywords of operators that send an integer to output','type': 'string'}
            sleep = 5
            config_params['sleep'] = {'title': 'Sleep','description': \
                'Time before starting next processing step','type': 'integer'}
            separator = ';'
            config_params['separator'] = {'title': 'Separator','description': \
                'Separates fields of logging stream','type': 'string'}



def process(log_str) :

    # start custom process definition
    parse_result = None

    marker = tfp.read_dict(api.config.marker_map)
    operators = marker.keys()

    lines = log_str.splitlines()
    for l in lines :
        fields = l.split(api.config.separator)
        for m in  marker.keys() :
            if fields[2].strip() == m  :
                pat = f'.*<{marker[m]}><(\d+)>'
                match_key = re.match(f'.*<{marker[m]}><(\d+)>',fields[3])
                if match_key :
                    index = match_key.group(1)
                    api.send(outports[1]['name'],str(index))
                    api.send(outports[2]['name'],int(index))


inports = [{"name":"log","type":"string","description":"Input log statements"}]
outports = [{"name":"log","type":"string","description":"Output of unchanged log statements"},\
            {"name":"string_string","type":"string","description":"Control string"},\
            {"name":"index","type":"int64","description":"Control index"}]

def call_on_input(log) :
    api.send(outports[0]['name'], log)
    process(log)

#api.set_port_callback(inports[0]['name'], call_on_input)

def main() :
    config = api.config
    config.separator = ';'
    config.sleep = 1
    config.marker_map = 'csv_to_df : BATCH ENDED, dropColumns : BATCH IN PROCESS'

    test_str = r"""10:20:06 ; INFO ; csv_to_df ; Process started
10:20:06 ; INFO ; csv_to_df ; Filename: AT_NO2.csv index: 1 count: 2 endofSeq: True
10:20:06 ; DEBUG ; csv_to_df ; Process ended: <BATCH ENDED><2> - 0m 0.178s ( 0: 0:0.2)
10:20:06 ; DEBUG ; csv_to_df ; Process ended: <BATCH IN PROCESS> <1>- 0m 0.178s ( 0: 0:0.2)
10:20:06 ; INFO ; dropColumns ; Process started
10:20:06 ; DEBUG ; dropColumns ; Columns: Index(['country_code', 'pollutant', 'x', 'y', 'coordsys', 'datetime', 'value', 'altitude'], dtype='object')
10:20:06 ; DEBUG ; dropColumns ; Process ended: <BATCH IN PROCESS> <2>- 0m 0.178s ( 0: 0:0.2)
10:20:06 ; DEBUG ; dropColumns ; Past process steps: ['EEA_pollution_getdata', 'dropColumns']
"""
    api.call(config=config, str = test_str)


if __name__ == '__main__':
    main()
    #gs.gensolution(os.path.realpath(__file__), config, inports, outports)
        




import sdi_utils.gensolution as gs
import sdi_utils.set_logging as slog
import sdi_utils.tprogress as tp

import os

try:
    api
except NameError:
    class api:
        class config:

            ## Meta data
            tags = {'python36': ''}  # tags that helps to select the appropriate container
            operator_description = 'Custom Operator Template'
            operator_description_long='Template Operator that provides the framework for a custom operator that includes ' \
                                      'all the information needed for generating the descriptive json-files and the ' \
                                      'README.md.'
            version = "0.0.1"  # for creating the manifest.json
            add_readme = dict()
            add_readme["References"] ="[Download template](https://raw.githubusercontent.com/thhapke/gensolution/master/diutil/customOperatorTemplate.py)"

            config_params = dict()
            ## config paramter
            debug_mode = True
            config_params['debug_mode'] = {'title': 'Debug mode',
                                           'description': 'Sending debug level information to log port',
                                           'type': 'boolean'}
            var1 = 'foo'
            config_params['var1'] = {'title': 'Variable1', 'description': 'Variable 1 for test', 'type': 'string'}
            var2 = 'bar'
            config_params['var2'] = {'title': 'Variable2', 'description': 'Variable 2 for test', 'type': 'string'}

        class Message:
            def __init__(self,body = None,attributes = ""):
                self.body = body
                self.attributes = attributes

        def send(port,msg) :
            if isinstance(msg,api.Message) :
                print('Port: ', port)
                print('Attributes: ', msg.attributes)
                print('Body: ', str(msg.body))
            else :
                print(str(msg))
            return msg

        def set_port_callback(port, callback) :
            default_msg = api.Message(attributes={'name':'doit'},body = 3)
            callback(default_msg)

        def call(config,msg):
            api.config = config
            return process(msg)


def process(msg):

    att_dict = msg.attributes
    att_dict['operator'] = 'customOperatorTemplate'
    if api.config.debug_mode == True:
        logger, log_stream = slog.set_logging(att_dict['operator'], loglevel='DEBUG')
    else:
        logger, log_stream = slog.set_logging(att_dict['operator'], loglevel='INFO')
    logger.info("Process started")
    time_monitor = tp.progress()


    result = ''
    logger.debug('Start Process Function')
    logger.debug('Start time: ' + time_monitor.get_start_time())
    for i in range (0,msg.body) :
        result += str(i) + ':' + api.config.var1 + ' - ' + api.config.var2 + '    '
    logger.debug('End of Process Function')
    logger.debug('End time: ' + time_monitor.elapsed_time())
    return api.Message(attributes={'name':'concat','type':'str'},body=result),log_stream.getvalue()

inports = [{"name":"input","type":"message","description":"Input data"}]
outports = [{"name":"output","type":"message","description":"Output data"},{"name":"log","type":"string","description":"Logging"}]

def call_on_input(msg) :
    new_msg, log = process(msg)
    api.send(outports[0]['name'],new_msg)
    api.send(outports[1]['name'],log)

#api.set_port_callback('input', call_on_input)

def main() :
    print('Test: Default')
    api.set_port_callback(inports[0]['name'], call_on_input)

    print('Test: config')
    config = api.config
    config.var1 = 'own foo'
    config.var12 = 'own bar'
    test_msg = api.Message(attributes={'name':'test1'},body =4)
    new_msg, log = api.call(config,test_msg)
    print('Attributes: ', new_msg.attributes)
    print('Body: ', str(new_msg.body))
    print('Logging: ')
    print(log)
    gs.gensolution(os.path.realpath(__file__), config, inports, outports,override_readme=True)



if __name__ == '__main__':
    main()
